@@ -503,7 +503,7 @@ class Interpreter(object):
             value = self.assigner.assign(value, target)
         stmt = ir.Assign(value=value, target=target, loc=self.loc)
         self.current_block.append(stmt)
-        self.definitions[name].append(value)
+        self.definitions[target.name].append(value)
 
     def get(self, name):
         """
@@ -576,7 +576,8 @@ class Interpreter(object):
         # then index the tuple-like object to extract the values
         for i, st in enumerate(stores):
             expr = ir.Expr.static_getitem(self.get(tupleobj),
-                                          index=i, loc=self.loc)
+                                          index=i, index_var=None,
+                                          loc=self.loc)
             self.store(expr, st)
 
     def op_BUILD_SLICE(self, inst, start, stop, step, res, slicevar):
@@ -1012,6 +1013,9 @@ class Interpreter(object):
     def op_BINARY_POWER(self, inst, lhs, rhs, res):
         self._binop('**', lhs, rhs, res)
 
+    def op_BINARY_MATRIX_MULTIPLY(self, inst, lhs, rhs, res):
+        self._binop('@', lhs, rhs, res)
+
     def op_BINARY_LSHIFT(self, inst, lhs, rhs, res):
         self._binop('<<', lhs, rhs, res)
 
@@ -1051,6 +1055,9 @@ class Interpreter(object):
     def op_INPLACE_POWER(self, inst, lhs, rhs, res):
         self._inplace_binop('**', lhs, rhs, res)
 
+    def op_INPLACE_MATRIX_MULTIPLY(self, inst, lhs, rhs, res):
+        self._inplace_binop('@', lhs, rhs, res)
+
     def op_INPLACE_LSHIFT(self, inst, lhs, rhs, res):
         self._inplace_binop('<<', lhs, rhs, res)
 
