@@ -23,7 +23,7 @@
  */
 
 static void
-pyobject_dtor(void *ptr, void* info) {
+pyobject_dtor(void *ptr, size_t size, void* info) {
     PyGILState_STATE gstate;
     PyObject *ownerobj = info;
 
@@ -417,4 +417,4 @@ init_nrt_python_module(PyObject *module)
     if (PyType_Ready(&MemInfoType))
         return -1;
     return 0;
-}
\ No newline at end of file
+}
