@@ -4,12 +4,12 @@ Definition of implementations for the `operator` module.
 
 import operator
 
-from numba.targets.imputils import implement, Registry
+from numba.targets.imputils import Registry
 from numba.targets import builtins
 from numba import types, utils
 
 registry = Registry()
-register = registry.register
+lower = registry.lower
 
 
 # Redirect the implementation of operator module functions to the
@@ -18,8 +18,7 @@ register = registry.register
 def map_operator(name, inplace_name, op):
     op_func = getattr(operator, name)
 
-    @register
-    @implement(op_func, types.VarArg(types.Any))
+    @lower(op_func, types.VarArg(types.Any))
     def binop_impl(context, builder, sig, args):
         impl = context.get_function(op, sig)
         return impl(builder, args)
@@ -27,8 +26,7 @@ def map_operator(name, inplace_name, op):
     if inplace_name:
         op_func = getattr(operator, inplace_name)
 
-        @register
-        @implement(op_func, types.VarArg(types.Any))
+        @lower(op_func, types.VarArg(types.Any))
         def binop_inplace_impl(context, builder, sig, args):
             first = sig.args[0]
             if first.mutable:
