@@ -402,7 +402,9 @@ class TestCache(TestCase):
         shutil.rmtree(self.tempdir)
 
     def import_module(self):
-        # Import a fresh version of the test module
+        # Import a fresh version of the test module.  All jitted functions
+        # in the test module will start anew and load overloads from
+        # the on-disk cache if possible.
         old = sys.modules.pop(self.modname, None)
         if old is not None:
             # Make sure cached bytecode is removed
@@ -444,7 +446,9 @@ class TestCache(TestCase):
         pass
 
     def run_in_separate_process(self):
-        # Cached functions can be run from a distinct process
+        # Cached functions can be run from a distinct process.
+        # Also stresses issue #1603: uncached function calling cached function
+        # shouldn't fail compiling.
         code = """if 1:
             import sys
 
@@ -452,6 +456,7 @@ class TestCache(TestCase):
             mod = __import__(%(modname)r)
             assert mod.add_usecase(2, 3) == 6
             assert mod.add_objmode_usecase(2, 3) == 6
+            assert mod.outer_uncached(3, 2) == 2
             assert mod.outer(3, 2) == 2
             packed_rec = mod.record_return(mod.packed_arr, 1)
             assert tuple(packed_rec) == (2, 43.5), packed_rec
@@ -513,6 +518,15 @@ class TestCache(TestCase):
         mod = self.import_module()
         self.assertPreciseEqual(mod.inner(3, 2), 6)
         self.check_cache(2)  # 1 index, 1 data
+        # Uncached outer function shouldn't fail (issue #1603)
+        f = mod.outer_uncached
+        self.assertPreciseEqual(f(3, 2), 2)
+        self.check_cache(2)  # 1 index, 1 data
+        mod = self.import_module()
+        f = mod.outer_uncached
+        self.assertPreciseEqual(f(3, 2), 2)
+        self.check_cache(2)  # 1 index, 1 data
+        # Cached outer will create new cache entries
         f = mod.outer
         self.assertPreciseEqual(f(3, 2), 2)
         self.check_cache(4)  # 2 index, 2 data
@@ -524,8 +538,12 @@ class TestCache(TestCase):
         mod = self.import_module()
         self.assertPreciseEqual(mod.outer(3, 2), 2)
         self.check_cache(4)  # 2 index, 2 data
+        self.assertPreciseEqual(mod.outer_uncached(3, 2), 2)
+        self.check_cache(4)  # same
+        mod = self.import_module()
         f = mod.inner
         self.assertPreciseEqual(f(3, 2), 6)
+        self.check_cache(4)  # same
         self.assertPreciseEqual(f(3.5, 2), 6.5)
         self.check_cache(5)  # 2 index, 3 data
 
@@ -589,6 +607,7 @@ class TestCache(TestCase):
         mod = self.import_module()
         mod.add_usecase(2, 3)
         mod.add_objmode_usecase(2, 3)
+        mod.outer_uncached(2, 3)
         mod.outer(2, 3)
         mod.record_return(mod.packed_arr, 0)
         mod.record_return(mod.aligned_arr, 1)
