@@ -167,11 +167,18 @@ def ldexp(x, e):
     return math.ldexp(x, e)
 
 
+def get_constants():
+    return math.pi, math.e
+
+
 class TestMathLib(TestCase):
 
     def setUp(self):
         self.ccache = CompilationCache()
 
+    def test_constants(self):
+        self.run_nullary_func(get_constants, no_pyobj_flags)
+
     def run_unary(self, pyfunc, x_types, x_values, flags=enable_pyobj_flags,
                   prec='exact', **kwargs):
         for tx, vx in zip(x_types, x_values):
@@ -198,8 +205,11 @@ class TestMathLib(TestCase):
     def check_predicate_func(self, pyfunc, flags=enable_pyobj_flags):
         x_types = [types.int16, types.int32, types.int64,
                    types.uint16, types.uint32, types.uint64,
-                   types.float32, types.float32, types.float64, types.float64]
-        x_values = [0, 0, 0, 0, 0, 0, float('inf'), 0.0, float('inf'), 0.0]
+                   types.float32, types.float32, types.float32,
+                   types.float64, types.float64, types.float64]
+        x_values = [0, 0, 0, 0, 0, 0,
+                    float('inf'), 0.0, float('nan'),
+                    float('inf'), 0.0, float('nan')]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
     def test_sin(self, flags=enable_pyobj_flags):
