@@ -38,8 +38,12 @@ class BasePYCCTest(TestCase):
 
     def setUp(self):
         self.tmpdir = static_temp_directory('test_pycc')
+        # Make sure temporary files and directories created by
+        # distutils don't clutter the top-level /tmp
+        tempfile.tempdir = self.tmpdir
 
     def tearDown(self):
+        tempfile.tempdir = None
         # Since we're executing the module-under-test several times
         # from the same process, we must clear the exports registry
         # between invocations.
